#pragma once


#include <Demos/SDLGame.hpp>
#include "JUI/Widgets/Scene.hpp"
#include "JUI/Widgets/Rect.hpp"
#include "Tools/gizmo.hpp"
#include "JUI/Bezier.hpp"
#include "JUI/Widgets/Text.hpp"
#include "JUI/Widgets/LayoutContainers.hpp"
#include "JUI/Widgets/Button.hpp"


class CurveToolEditor : public SDLGame
{
public:
    CurveToolEditor() : SDLGame() { }
    ~CurveToolEditor() override {}
    JUI::Scene *gui;
    gizmo p0 = gizmo({100, 100});
    gizmo p1 = gizmo({200, 100});
    gizmo p2 = gizmo({100, 200});
    gizmo p3 = gizmo({200, 200});

    JUI::Rect* CreateMenu(const std::string& name, std::vector<std::string> selections)
    {
        auto* widget = new JUI::Button();
        widget->SetSize({60, 0, 0, 1});
        widget->SetText(name);
        widget->SetClipsDescendants(false);

        auto* submenu = new JUI::Rect(widget);
        submenu->SetSize({0, 0, 1, 1});
        submenu->SetAnchorPoint({0, 0});
        submenu->SetPosition({0, 0, 0, 1});
        submenu->SetClipsDescendants(false);
        auto* container = new JUI::VerticalListLayout(submenu);
        for (std::string element : selections)
        {
            auto* subselection = new JUI::Button(container);
            subselection->SetText(element);
            subselection->SetSize({0,20,1,0});
        }
        submenu->SetVisible(false);

        widget->MousePressed += [submenu](auto mouseBtnId){
            submenu->SetVisible(true);
        };

        return widget;
    }

    void Initialize() override
    {
        SDLGame::Initialize();
        JUI::Load(this->renderer);

        gui = new JUI::Scene(window, renderer);

        auto* topbar = new JUI::Rect(gui);
        topbar->SetSize({0,20,1,0});

        auto* alignedContainer = new JUI::HorizontalListLayout(topbar);
        //alignedContainer->

        auto* file = CreateMenu("File", {"New", "Open", "Save"});
        file->SetParent(alignedContainer);
        auto* edit = CreateMenu("Edit", {});
        edit->SetParent(alignedContainer);

        alignedContainer->ApplyLayout();
    }

    void Update(double dt) override
    {
        JUI::Update(dt);
        gui->Update(dt);

        p0.Update(); p1.Update();
        p2.Update(); p3.Update();
    }

    void Draw() override
    {
        gui->Draw();

        SDL_Color red {255,   0,   0, 255};
        SDL_Color blu {128, 128, 255, 255};

        JUI::DrawCurve(renderer, red, p0.position, p1.position, p2.position, p3.position, 3);
        JUI::DrawCurve(renderer, blu, p0.position, p1.position, p2.position, p3.position, 30);

        p0.Draw(renderer);
        p1.Draw(renderer);
        p2.Draw(renderer);
        p3.Draw(renderer);

    }

};