/* @file       Font.hpp
 * @desc       Class Wrapper for SDL_TTF Font and SDL_FontCache functionality
 * @auth       Josh O'Leary
 * @revision   1
 * @edited     2023-08-15
 * @copyright  (C) 2023 Conarium Software
 * @license    Unlicense - Public Domain
 */
#pragma once

#include <SDL.h>
#include <SDL_main.h>
#include <SDL_ttf.h>

#include <SDL_FontCache.h>

#include <cstdlib>
#include <cstdio>
#include <string>

typedef enum
{
	TextRenderSolid,
	TextRenderShaded,
	TextRenderBlended
} TextRenderMethod;


namespace JUI 
{
    ///
    ///
    ///
    class Font {
#pragma region Interface
    public:
#pragma region Constructors
        Font() {}
        Font(SDL_Renderer *target, std::string font, int pt_size);
        ~Font();
#pragma endregion

#pragma region Member Methods
    FC_Font* GetCachedFontPtr() const;
#pragma region Getters
    FC_FilterEnum GetFilterMode() const;
    uint16_t GetLineHeight() const;
    uint16_t GetHeight(const std::string& formatted_text, ...) const;
    uint16_t GetWidth(const std::string& formatted_text, ...)  const;
    int GetAscent(const std::string& formatted_text, ...)      const;
    int GetBaseline()    const;
    int GetSpacing()     const;
    int GetLineSpacing() const;
    uint16_t  GetMaxWidth()     const;
    SDL_Color GetDefaultColor() const;

#pragma endregion
#pragma region Setters
    void SetFilterMode(FC_FilterEnum filter);
    void SetSpacing(int letter_spacing);
    void SetLineSpacing(int line_spacing);
    void SetDefaultColor(SDL_Color color);
#pragma endregion

#pragma endregion

#pragma endregion
#pragma region Implementation
    FC_FilterEnum text_filter_mode;
    FC_Font *font_ptr{};
    private:


    protected:

#pragma endregion
    };


}