/// @file
/// @desc
/// @auth
/// @revision
/// @edited
/// @copyright
/// @license

#pragma once

#include <functional>

namespace JUI
{

    namespace Easing
    {
        using EasingStyleCallbackSignature = std::function<float(float, float, float)>;
        // Value changes with constant speed.
        float Default(float position, float start, float end);
        // Value change with constant speed
        float Linear(float position, float start, float end);


        namespace Quadratic {
            float In   (float position, float start, float end); // Accelerate initial values with a quadratic equation.
            float Out  (float position, float start, float end); // Decelerate ending values with a quadratic equation.
            float InOut(float position, float start, float end); // Accelerate initial and decelerate ending values with a quadratic equation.
        }

        namespace Cubic {
            float In   (float position, float start, float end); // Accelerate initial values with a cubic equation.
            float Out  (float position, float start, float end); // Decelerate ending values with a cubic equation.
            float InOut(float position, float start, float end); // Accelerate initial and decelerate ending values with a cubic equation.
        }

        namespace Quartic {
            float In   (float position, float start, float end); // Accelerate initial values with a quartic equation.
            float Out  (float position, float start, float end); // Decelerate ending values with a quartic equation.
            float InOut(float position, float start, float end); // Accelerate initial and decelerate ending values with a quartic equation.
        }

        namespace Quintic {
            float In   (float position, float start, float end); // Accelerate initial values with a quintic equation.
            float Out  (float position, float start, float end); // Decelerate ending values with a quintic equation.
            float InOut(float position, float start, float end); // Accelerate initial and decelerate ending values with a quintic equation.
        }

        namespace Sinusoidal {
            float In   (float position, float start, float end); // Accelerate initial values with a sinusoidal equation.
            float Out  (float position, float start, float end); // Decelerate ending values with a sinusoidal equation.
            float InOut(float position, float start, float end); // Accelerate initial and decelerate ending values with a sinusoidal equation.
        }

        namespace Exponential {
            float In   (float position, float start, float end); // Accelerate initial values with an exponential equation.
            float Out  (float position, float start, float end); // Decelerate ending values with an exponential equation.
            float InOut(float position, float start, float end); // Accelerate initial and decelerate ending values with an exponential equation.
        }

        namespace Circular {
            float In   (float position, float start, float end); // Accelerate initial values with a circular equation.
            float Out  (float position, float start, float end); // Decelerate ending values with a circular equation.
            float InOut(float position, float start, float end); // Accelerate initial and decelerate ending values with a circular equation.
        }

        namespace Bounce {
            float In   (float position, float start, float end); // Accelerate initial values with a "bounce" equation.
            float Out  (float position, float start, float end); // Decelerate ending values with a "bounce" equation.
            float InOut(float position, float start, float end); // Accelerate initial and decelerate ending values with a "bounce" equation.
        }

        namespace Elastic {
            float In   (float position, float start, float end); // Accelerate initial values with an "elastic" equation.
            float Out  (float position, float start, float end); // Decelerate ending values with an "elastic" equation.
            float InOut(float position, float start, float end); // Accelerate initial and decelerate ending values with an "elastic" equation.
        }

        namespace Back {
            float In   (float position, float start, float end); // Accelerate initial values with a "back" equation.
            float Out  (float position, float start, float end); // Decelerate ending values with a "back" equation.
            float InOut(float position, float start, float end); // Accelerate initial and decelerate ending values with a "back" equation.
        }
    }

    template <typename... Args>
    using delegate = std::function<void(Args...)>;

    struct Tween {
#pragma region Initializers
        float Start;
        float Goal;
        float Runtime;
        Easing::EasingStyleCallbackSignature EasingStyle = Easing::Default;
        delegate<float> Callback;
#pragma endregion

#pragma region Methods
        void Update(double delta);
        float Step(float progress) const;
        void Stop();
        float GetProgress() const;
#pragma endregion

        float progress = 0.f;
        bool done = false; // TODO: Candidate rename "Dead"
    };
}


