#pragma once


#include <JUI/common.h>
#include <JUI/Types/UDim.hpp>

namespace JUI {

    // Funky Coordinate system purpose-built for screen positioning
    // Contains an X and Y UDim, each containing an integer "Pixels", and fractional "Scale" (0-1) value.
    // GUI Elements use this type to describe their sizing in relation to their parent.
    // @see UDim.hpp
    class UDim2 {

    public:
        UDim X;
        UDim Y;

#pragma region Operators
        UDim2 operator+(const UDim2& rhs) const;
        UDim2 operator-(const UDim2& rhs) const;
        UDim2 operator*(float rhs) const;
        UDim2 operator/(float rhs) const;
        UDim2 &operator=(const UDim2 &) = default;
#pragma endregion

#pragma region Constructors
        UDim2();
        UDim2(int px, int py, float sx, float sy);
        UDim2(v2f pixels, v2f scale);
        UDim2(UDim x, UDim y);
        UDim2(const UDim2 &u) : X(u.X), Y(u.Y) {}
        static UDim2 FromPixels(int x, int y);
        static UDim2 FromScale(float x, float y);

#pragma endregion

#pragma region Methods
        v2f GetScale() const;
        v2f GetPixels() const;
#pragma endregion

    protected:
    private:
    };
}