#pragma once

#include <JUI/common.h>

namespace JUI {
    namespace TextAlignment {
        enum class Horizontal { LEFT, CENTER, RIGHT };
        enum class Vertical { TOP, CENTER, BOTTOM };
    }

    class TextBase {
    public:
        TextBase();
        std::string GetText() const;
        SDL_Color   GetTextColor() const;
        SDL_Color   GetTextOutlineColor() const;
        JUI::Font*  GetFont() const;
        TextAlignment::Vertical   GetVerticalTextAlignment()   const;
        TextAlignment::Horizontal GetHorizontalTextAlignment() const;
        bool GetWordWrap() const;
        v2i  GetTextBounds() const;
        v2f  GetTextPosition() const;

        void SetWordWrap(bool wrap);
        void SetText(std::string text);
        void SetTextColor(SDL_Color c);
        void SetTextOutlineColor(SDL_Color c);
        void SetFont(JUI::Font* font);
        void SetVerticalTextAlignment(TextAlignment::Vertical align);
        void SetHorizontalTextAlignment(TextAlignment::Horizontal align);
        void Center();

        void Draw(SDL_Renderer* target, v2f base_coords, v2f bounds);
    protected:
        SDL_Color text_color;
        float text_outline;
        JUI::Font* font;
        std::string text = "Sample Text";
        bool word_wrap = false;
        SDL_Color outline_color;
        TextAlignment::Horizontal h_text_alignment = TextAlignment::Horizontal::LEFT;
        TextAlignment::Vertical v_text_alignment = TextAlignment::Vertical::TOP;
        bool rich_text_enabled;
    private:
    };

}