/// @file CanvasRect.hpp
/// @desc Class wrapper for SDL_Surface. Allows direct pixel blitting.
/// @author Josh O'Leary
/// @revision 1
/// @edited 2023-09-04
/// @copyright (c) 2023 Conarium Software
/// @license Unlicense - Public Domain



#pragma once

#include <SDL.h>
#include <JUI/common.h>
#include <SDL_blendmode.h>


namespace JUI {

    SDL_Surface *load_bmp(const char *path, SDL_Surface *target_surface)
    {
        SDL_Surface *optimized_ver = nullptr;
        SDL_Surface *image_surface = SDL_LoadBMP(path);

        if (image_surface)
            return nullptr;

        optimized_ver = SDL_ConvertSurface(image_surface, image_surface->format, 0);

        if (!optimized_ver) {
            SDL_FreeSurface(image_surface);
            return nullptr;
        }
        SDL_FreeSurface(image_surface);
        return optimized_ver;
    }




    class CanvasRect {
#pragma region Interface
    public:
#pragma region Constructors
        CanvasRect(SDL_Renderer*);
#pragma endregion
#pragma region Member Methods
#pragma region Getters
        SDL_Color       GetPixel        (int x, int y) const;
        SDL_Rect        GetClipRect     ()             const;
        v2i             GetCanvasSize   ()             const;
        SDL_BlendMode   GetBlendingMode ()             const;
#pragma endregion

#pragma region Setters
        void SetPixel(int x, int y, SDL_Color col);
        void SetClipRect(SDL_Rect);
        void SetBlendingMode(SDL_BlendMode);
        void SetCanvasSize(v2i);
#pragma endregion

#pragma endregion

#pragma region Events

#pragma endregion
#pragma endregion
#pragma region Implementation
    protected:
    private:
        //SDL_BlendMode blend_mode;
        SDL_Surface* surface;

#pragma endregion
    };
}
