#pragma once

#include <JUI/Widgets/Widget.hpp>

namespace JUI {
    // Base Layout Widget
    // TODO: Implement Sort Order Criterion
    // TODO: Implement LayoutDirection
    class LayoutContainerWidget : public Widget {
    public:
        LayoutContainerWidget();
        LayoutContainerWidget(Widget* parent);
        ~LayoutContainerWidget() override;
        void Update(double delta) override;
        void Draw(SDL_Renderer *target) override;
        virtual void ApplyLayout();

        v2f GetAbsoluteSize() const override { return parent->GetAbsoluteSize(); }
        v2f GetAbsolutePosition() const override { return parent->GetAbsolutePosition(); }
    protected:
        UDim padding = {0, 0.0f};
    private:
    };

    // Lays child widgets out in a vertical list.
    class VerticalListLayout : public LayoutContainerWidget {
    public:
        VerticalListLayout() : LayoutContainerWidget() {}
        VerticalListLayout(Widget* parent) : VerticalListLayout()
        {
            this->SetParent(parent);
        }
        ~VerticalListLayout() override {}

        void Update(double delta) override;
        void Draw(SDL_Renderer *target) override;
        void SetPadding(UDim padding);
        UDim GetPadding();
        void ApplyLayout() override;
    private:
    };


    enum class Align { LEFT, RIGHT};

    class HorizontalListLayout : public LayoutContainerWidget {
    public:

        HorizontalListLayout() : LayoutContainerWidget() {}
        HorizontalListLayout(Widget* parent) : HorizontalListLayout()
        {
            this->SetParent(parent);
        }
        ~HorizontalListLayout() override;
        void ApplyLayout() override;
        void Update(double delta);
        void Draw(SDL_Renderer *target);
        void SetPadding(UDim padding);
        UDim GetPadding();
    protected:
        Align align = Align::LEFT;
    private:
    };

    enum Orientation {
        VERTICAL, HORIZONTAL
    };

    class GridLayout : public LayoutContainerWidget {
        GridLayout() {}

        ~GridLayout() override {}

    public:
        void ApplyLayout() override {

        }

    private:
        UDim2 cell_padding;
        UDim2 cell_size;
        int max_cells;
        UDim2 margin;
    };

    class Slider : public Widget {
    public:
    private:
        bool smooth_increment;
        float min;
        float max;
        float value;
        float increment;
        Orientation orientation;
    };
}