
#pragma once

#include <JUI/Widgets/Widget.hpp>
#include <SDL_pixels.h>
#include <JUI/Types/UDim2.hpp>
#include <JUI/common.h>

namespace JUI {
    /* The Rect Widget renders a rectangle and also serves
     * as a versatile container for laying out other widgets.
     */

    // TODO: Refactor into RectBase class and Rect widget class
    class Rect : public Widget {
#pragma region Interface
    public:
#pragma region Constructors
        Rect();
        Rect(Widget* parent);
        ~Rect() override {}
#pragma endregion
#pragma region Event
        Event<v2f> MouseEnter;
        Event<v2f> MouseExit;
        Event<v2f> InputPressed;
        Event<v2f> InputReleased;
        Event<MouseButtonID> MousePressed;
        Event<MouseButtonID> MouseReleased;
        Event<v2f> TouchPressed;
        Event<v2f> TouchReleased;
#pragma endregion
#pragma region Member Methods
        void Draw  (SDL_Renderer* target) override;
        void Update(double delta)         override;
        bool IsMouseInside() const;
        //bool IsOnScreen() {}
        //bool IsPointInside(SDL_Point ref) {}
#pragma region Getters
        SDL_Color GetBackgroundColor  () const;
        SDL_Color GetBorderColor      () const;
        float     GetBorderWidth      () const;
        bool      GetClipsDescendants () const;
        float     GetBorderRadius     () const;
        bool      GetAutoSizing       () const {}

#pragma endregion
#pragma region Setters
        void SetAutoSizing(bool enabled);
        void SetBackgroundColor(SDL_Color);
        void SetBorderColor(SDL_Color);
        void SetBorderWidth(float pixels);
        void SetClipsDescendants(bool enabled);
        void SetBorderRadius(float degrees);
        v2f GetMouseCoordinates() const {
            int x = 0;
            int y = 0;
            SDL_GetMouseState(&x, &y);
            v2f mc{x, y};
            return mc;
        };
#pragma endregion
#pragma endregion
#pragma region Events
#pragma endregion
#pragma endregion
#pragma region Internals
    protected:
        float border_width = 1.0f; // Pixel thickness of the border lines
        SDL_Color bg_color    {128,128,128,128};  // {RGBA} format color struct
        SDL_Color border_color{192,192,192,255};  // {RGBA} format color struct
        bool clips_descendants = false;  // Controls if child objects can render outside of the rectangle bounds.
        float border_radius = 0; // Curves the rectangle corners by n degrees
        MouseButtonState prev_state = {false, false, false};
        bool was_clicked = false;
        bool mouse_hover = false;
    private:

#pragma endregion
    };

}
