/// @file       Scene.hpp
/// @desc       A Scene is the root of a JUI object tree.
/// @auth       Josh O'Leary
/// @revision   1
/// @edited     2023-08-13
/// @copyright  (C) 2023 Conarium Software
/// @license    Unlicense - Public Domain

#pragma once

#include <SDL.h>
#include <JUI/Widgets/Widget.hpp>

namespace JUI {

    // Scene
    class Scene : public Widget {
    public:
#pragma region Interface
#pragma region Constructors
        Scene();
        Scene(SDL_Window *wnd, SDL_Renderer *rend);
        ~Scene() override {}
#pragma endregion
#pragma region Events

#pragma endregion
#pragma region Member Methods
        void Draw(SDL_Renderer *target) override;
#pragma region Getters
        Widget* GetFocusedWidget() const { return focused_widget; }
        [[nodiscard]] v2f GetAbsolutePosition() const override; // Returns the size of the window
        [[nodiscard]] v2f GetAbsoluteSize() const override; // Returns {0, 0}
#pragma endregion
#pragma region Setters
        void SetFocusedWidget(Widget* widget)
        {
            focused_widget = widget;
        }
#pragma endregion
#pragma endregion
#pragma endregion

#pragma region Implementation

        void KeyPressed(SDL_KeyboardEvent e) { }
        void KeyReleased(SDL_KeyboardEvent e) { }

        // Direct Call???
        void Draw();

        void Update(double delta) override;

    protected:
    private:
        SDL_Window *window;
        SDL_Renderer *renderer;
        Widget* focused_widget;
#pragma endregion
    };





}
