#pragma once

#include <JUI/Widgets/Widget.hpp>
#include <JUI/Widgets/Rect.hpp>
#include <JUI/Widgets/Text.hpp>

namespace JUI {

    class DraggableElement {
    public:
        bool GetDrag() const { return dragging; }
        virtual void SetDrag(bool b) {
            this->dragging = b;
        }
    protected:
        bool dragging = false;
        v2f initial_drag_offset = {0,0};
    private:
    };

    class Window : public Rect , public DraggableElement  {
    public:
#pragma region Constructors
        Window();
        Window(Widget*);
#pragma endregion
#pragma region Getters
        v2i GetMaxDimensions() const;
        v2i GetMinDimensions() const;
        v2i GetCurrentDimensions() const;
        int GetTitlebarHeight() const;
        bool GetDraggable() const;
        bool GetResizable() const;
        std::string GetTitle() const;
#pragma endregion
#pragma region Setters
        void SetMaxDimensions(const v2i& dims);
        void SetMinDimensions(const v2i& dims);
        void SetTitlebarHeight(int);
        void SetDraggable(bool enabled);
        void SetResizable(bool enabled);
        void SetTitle(const std::string&);

        void Update(double) override;
        void Draw(SDL_Renderer*) override;

        void SetDrag(bool d) override {
            DraggableElement::SetDrag(d);
            this->initial_drag_offset = this->GetMouseCoordinates() - this->GetAbsolutePosition();
        }
#pragma endregion
    protected:
        JUI::Rect* Topbar;
        JUI::Rect* Viewport;
        JUI::Text* TitleLabel;
        std::string title = "JUI Window";
        bool resizable = true;
        bool resizing = false;
        bool draggable = true;
        int titlebar_height = 16;
        v2i max_dims;
        v2i min_dims;
    private:
    };



}