/// @file       gizmo.hpp
/// @desc       Draggable gizmo class for manipulating points in 2D via the mouse
/// @author     Joshua O'Leary
/// @edited     25 October 2023
/// @version    1
/// @copyright  (c) Conarium Software 2023
/// @license    Unlicense - Public Domain

#pragma once

#include "JUI/common.h"
#include <SDL2_gfxPrimitives.h>

using namespace JUI;

class gizmo {
public:
    gizmo() : position{0,0}
    {
        dragging = false;
        mouse_hover = false;
    }

    gizmo(const v2f& pos)
    {
        dragging = false;
        mouse_hover = false;
        position = pos;
    }
    bool dragging;
    bool mouse_hover;
    v2f position;
    float mouse_click_range = 4.0f;
    MouseButtonState prev_state;
    void Update()
    {
        v2f m_pos = JUI::GetMousePosition();

        mouse_hover = (glm::distance(m_pos, position) < mouse_click_range);

        int x = 0;
        int y = 0;
        Uint32 mouse_btn_bitmask = SDL_GetMouseState(&x, &y);


        MouseButtonState state{};
        state.LMB = mouse_btn_bitmask & SDL_BUTTON(1);
        state.MMB = mouse_btn_bitmask & SDL_BUTTON(2);
        state.RMB = mouse_btn_bitmask & SDL_BUTTON(3);


        // TODO: Encapsulate this mess into InputService?
        if (!prev_state.LMB && state.LMB && mouse_hover)
        {
            dragging = true;
        }

        if (prev_state.LMB && !state.LMB && dragging)
        {
            dragging = false;
        }

        if (dragging)
            position = JUI::GetMousePosition();

        prev_state = state;

    }
    void Draw(SDL_Renderer* renderer)
    {
        int dot_size = 2;

        if (mouse_hover) { dot_size = 4; }

        filledCircleRGBA(renderer, position.x, position.y, dot_size, 0, 255, 0, 255);
    }
protected:
private:
};