#include "fmt/format.h"
#include <SDL_platform.h>
#include "JUI/NumberFormatter.h"
#include "JUI/Widgets/LayoutContainers.hpp"
#include "JUI/Widgets/Scene.hpp"
#include "JUI/Widgets/Rect.hpp"
#include "JUI/Types/Font.hpp"
#include "JUI/Widgets/Text.hpp"
#include "JUI/common.h"
#include "Demos/MockMenu/JUIDemoGame.hpp"
#include "JUI/Widgets/Window.h"

void JUIDemoGame::Initialize() {
    SDLGame::Initialize();
    JUI::Load(this->renderer);

    Gui = new JUI::Scene(window, renderer);

    //auto* test_window = new JUI::Window();

#pragma region Gui Event Passthrough Callbacks
    // This is an example of how to integrate SDL event with JUI
    // Perhaps a refactor is in order to make it nicer?
    // AKA Can we simplify life for the end-users?
    this->KeyPressed    += [this](SDL_KeyboardEvent event) {
        Gui->KeyPressed(event);
    };
    this->KeyReleased   += [this](SDL_KeyboardEvent event) {
        Gui->KeyReleased(event);
    };
    this->MousePressed  += [this](SDL_MouseButtonEvent e) {

    };
    this->MouseReleased += [this](SDL_MouseButtonEvent e) {

    };
    JUI::MouseMoved  += [this](JUI::MouseMotionEvent e) {
        std::cout << e.Position.x << ", " << e.Position.y
        << ", " << e.Delta.x << ", " << e.Delta.y << std::endl;
    };
    this->WindowChanged += [this](SDL_WindowEvent event) {
        switch (event.event) {
            // TODO: Add specific delegates for relevant window events
            case SDL_WINDOWEVENT_SHOWN:
                SDL_Log("Window %d shown", event.windowID);
                break;
            case SDL_WINDOWEVENT_HIDDEN:
                SDL_Log("Window %d hidden", event.windowID);
                break;
            case SDL_WINDOWEVENT_EXPOSED:
                SDL_Log("Window %d exposed", event.windowID);
                break;
            case SDL_WINDOWEVENT_MOVED:
                SDL_Log("Window %d moved to %d,%d",
                        event.windowID, event.data1,
                        event.data2);
                break;
            case SDL_WINDOWEVENT_RESIZED:
                SDL_Log("Window %d resized to %dx%d",
                        event.windowID, event.data1,
                        event.data2);
                break;
            case SDL_WINDOWEVENT_SIZE_CHANGED:
                SDL_Log("Window %d size changed to %dx%d",
                        event.windowID, event.data1,
                        event.data2);
                break;
            case SDL_WINDOWEVENT_MINIMIZED:
                SDL_Log("Window %d minimized", event.windowID);
                break;
            case SDL_WINDOWEVENT_MAXIMIZED:
                SDL_Log("Window %d maximized", event.windowID);
                break;
            case SDL_WINDOWEVENT_RESTORED:
                SDL_Log("Window %d restored", event.windowID);
                break;
            case SDL_WINDOWEVENT_ENTER:
                SDL_Log("Mouse entered window %d",
                        event.windowID);
                break;
            case SDL_WINDOWEVENT_LEAVE:
                SDL_Log("Mouse left window %d", event.windowID);
                break;
            case SDL_WINDOWEVENT_FOCUS_GAINED:
                SDL_Log("Window %d gained keyboard focus",
                        event.windowID);
                break;
            case SDL_WINDOWEVENT_FOCUS_LOST:
                SDL_Log("Window %d lost keyboard focus",
                        event.windowID);
                break;
            case SDL_WINDOWEVENT_CLOSE:
                SDL_Log("Window %d closed", event.windowID);
                break;
#if SDL_VERSION_ATLEAST(2, 0, 5)
            case SDL_WINDOWEVENT_TAKE_FOCUS:
                SDL_Log("Window %d is offered a focus", event.windowID);
                break;
            case SDL_WINDOWEVENT_HIT_TEST:
                SDL_Log("Window %d has a special hit test", event.windowID);
                break;
#endif
            default:
                SDL_Log("Window %d got unknown event %d",
                        event.windowID, event.event);
                break;
        }
    };

#pragma endregion

    auto *rect = new JUI::Rect();

    rect->SetName("YA NOTHIN");
    rect->SetBackgroundColor({64, 64, 64, 255});
    rect->SetBorderWidth(3);
    rect->SetBorderColor({255, 255, 255, 255});
    rect->SetSize(    {50_percent + 5_px, 50_percent});
    rect->SetPosition({50_percent, 50_percent});
    rect->SetAnchorPoint({0.5f, 0.5f});
    rect->SetParent(Gui);

    //rect->MousePressed += [rect] (MouseButtonID id) mutable { if (!rect->GetDrag()) rect->SetDrag(true); };
    //rect->MouseReleased += [rect](MouseButtonID id) mutable { if (rect->GetDrag()) rect->SetDrag(false); };

    auto* topbar = new JUI::Rect();

    topbar->SetName("Topbar");
    topbar->SetAnchorPoint({0.f, 1.f});
    topbar->SetSize({100_percent, 20_pixels});
    topbar->SetPosition({0_px, 0_px});
    topbar->SetPaddingTop(0_px);
    topbar->SetBackgroundColor({32,32,32,255});
    topbar->SetBorderWidth(3);
    topbar->SetParent(rect);

    auto* title = new JUI::Text();
    title->SetVerticalTextAlignment(TextAlignment::Vertical::CENTER);
    title->SetHorizontalTextAlignment(TextAlignment::Horizontal::CENTER);
    title->SetTextColor({255,255,255,255});
    title->SetText("Test Window");
    title->SetParent(topbar);

    auto* exit_btn = new JUI::Rect();

    exit_btn->SetParent(topbar);
    exit_btn->SetAnchorPoint({1.f, 0.f});
    exit_btn->SetSize({50_px, 100_percent});
    exit_btn->SetPosition({100_percent, 0_percent});

    exit_btn->MouseReleased += [rect](MouseButtonID e) {
        if (e == MouseButtonID::LMB) {
            rect->SetVisible(false);
        }
    };


    auto* exit_btn_label = new JUI::Text();
    exit_btn_label->SetVerticalTextAlignment(TextAlignment::Vertical::CENTER);
    exit_btn_label->SetHorizontalTextAlignment(TextAlignment::Horizontal::CENTER);
    exit_btn_label->SetTextColor({255, 0, 0, 255});
    exit_btn_label->SetText("X");
    exit_btn_label->SetParent(exit_btn);

    auto *rect2 = new JUI::Rect();

    rect2->SetParent(rect);
    rect2->SetName("NO TALENT");
    rect2->SetClipsDescendants(true);
    rect2->SetBackgroundColor({0, 0, 255, 255});
    rect2->SetSize({0, 0, 0.33f, 1.0f});
    rect2->SetPosition({0_percent, 0_percent});
    rect2->SetAnchorPoint({0.0f, 0.0f});

    // TODO: Should we Implement Default Font?
    auto *layout = new JUI::VerticalListLayout();
    layout->SetParent(rect2);

    // Textbox 1
    auto *tb1 = new JUI::Rect();
    tb1->SetParent(layout);
    tb1->SetSize({0, 30, 1.0f, 0.0f});
    tb1->SetBackgroundColor({128, 128, 128, 255});
    tb1->SetBorderRadius(2);
    auto _submit_press = tb1->MousePressed += [tb1](MouseButtonID id) {
        if (id == MouseButtonID::LMB) { tb1->SetBackgroundColor({64, 64, 64, 255}); }
    };
    auto _submit_confirm = tb1->MouseReleased += [tb1](MouseButtonID id) {
        if (id == MouseButtonID::LMB) { tb1->SetBackgroundColor({128, 128, 128, 255}); }
    };


    // TODO: Implement Button Widget?

    using namespace JUI::TextAlignment;

    auto *lb1 = new JUI::Text();

    lb1->SetParent(tb1);
    lb1->SetHorizontalTextAlignment(TextAlignment::Horizontal::CENTER);
    lb1->SetVerticalTextAlignment  (TextAlignment::Vertical::CENTER);
    lb1->SetTextColor({0, 0, 0, 255});
    lb1->SetText("YA NOTHIN");

    auto *tb2 = new JUI::Rect();

    tb2->SetParent(layout);
    tb2->SetSize({0, 50, 1.0f, 0.0f});
    tb2->SetBackgroundColor({0, 255, 255, 255});


    auto *lb2 = new JUI::Text();

    lb2->SetParent                 (tb2);
    lb2->SetHorizontalTextAlignment(TextAlignment::Horizontal::LEFT);
    lb2->SetVerticalTextAlignment  (TextAlignment::Vertical::TOP);
    lb2->SetTextColor              ({0, 0, 0, 255});
    lb2->SetText                   ("NO TALENT");


    //SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_INFORMATION, "JUI Widgets Demonstration", "Enjoy the widgets!!!", NULL);
}

void JUIDemoGame::Update(double delta) {
    SDLGame::Update(delta);

    Gui->Update(delta);
}

[[gnu::hot]]
void JUIDemoGame::Draw() {
    SDLGame::Draw();

    Gui->Draw(renderer);

    // Draw some debug info

    auto size = GetWindowSize();

    auto window_size_str = fmt::format     ("Window Size: X:{}, Y:{}", size.x, size.y);
    auto platform_ver    = SDL_GetPlatform ();
    auto ram             = SDL_GetSystemRAM();
    auto amt             = Trunicate     (ram) + " MB";
    auto cpu_ct          = std::to_string(SDL_GetCPUCount()) + " cores";
    auto err_hint        = std::string   (SDL_GetError());
    auto framerate       = fmt::format   ("FPS: {}", GetFramerate());


    draw_simple_text(this->renderer, window_size_str,   0, 0);
    draw_simple_text(this->renderer, platform_ver,      0, 14);
    draw_simple_text(this->renderer, amt,               0, 14*2);
    draw_simple_text(this->renderer, cpu_ct,            0, 14*3);
    draw_simple_text(this->renderer, err_hint,          0, 14*4);
    draw_simple_text(this->renderer, framerate,         0, 14*5);
}

void JUIDemoGame::Screenshot() {
    auto window_dims = GetWindowSize();
    const char* screenie_timestamp = "";
    SDL_Surface *screenie = SDL_CreateRGBSurface(0, window_dims.x, window_dims.y, 32, 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000);
    SDL_RenderReadPixels(renderer, nullptr, SDL_PIXELFORMAT_ARGB8888, screenie->pixels, screenie->pitch);
    SDL_SaveBMP(screenie, screenie_timestamp);
}
