#include <JUI/Bezier.hpp>

namespace JUI {


    void DrawCurve(SDL_Renderer *renderer, SDL_Color col, const v2f &p0, const v2f &p1, const v2f p2, const v2f p3,
                   int subdivisions) {
        // Render Curve
        v2f last = p0;
        v2f first = p3;

        // Draw lines between control points
        thickLineRGBA(renderer, p0.x, p0.y, p1.x, p1.y, 1, 128,128,128,255);
        thickLineRGBA(renderer, p1.x, p1.y, p2.x, p2.y, 1, 128,128,128,255);
        thickLineRGBA(renderer, p2.x, p2.y, p3.x, p3.y, 1, 128,128,128,255);

        for (int i = 0; i < subdivisions; i++) {
            float progress_normalized = (float)i / (float)subdivisions;
            auto interpolated =  Bezier(progress_normalized, p0, p1, p2, p3);
            thickLineRGBA(renderer, last.x, last.y, interpolated.x, interpolated.y, 2, col.r, col.g, col.b, col.a);
            last = interpolated;
        }

        // Make sure last segment of curve is drawn
        SDL_RenderDrawLine(renderer, last.x, last.y, first.x, first.y);

        // Draw Control Points on top of curve
        SDL_SetRenderDrawColor(renderer, 0, 255, 0, 255);
        SDL_RenderDrawPoint(renderer, p0.x, p0.y);
        SDL_RenderDrawPoint(renderer, p1.x, p1.y);
        SDL_RenderDrawPoint(renderer, p2.x, p2.y);
        SDL_RenderDrawPoint(renderer, p3.x, p3.y);
    }

    void DrawCurve(SDL_Renderer *renderer, SDL_Color col, const curve &c, int subdivisions) {
        JUI::DrawCurve(renderer, col, c.p0, c.p1, c.p2, c.p3, subdivisions);
    }

    v2f BezierNormal(float t, const v2f &p0, const v2f &p1, const v2f &p2, const v2f &p3) {
        auto deriv = BezierDerivative(t, p0, p1, p2, p3);
        return Normalized(v2f(-deriv.y, -deriv.x));
    }

    v2f BezierDerivative(float t, const v2f &p0, const v2f &p1, const v2f &p2, const v2f &p3) {
        return 3 * Square(1 - t) * (p1 - p0) + 6 * (1 - t) * t * (p2 - p1) + 3 * Square(t) * (p3 - p2);
    }
}