#include <JUI/Widgets/Base/TextBase.hpp>

namespace JUI {
    TextBase::TextBase() {
        text_color = {255, 255, 255, 255};
        outline_color = {0, 0, 0, 255};
        text_outline = 1.f;
        font = JUI::GetDefaultFont();
    }

    v2i TextBase::GetTextBounds() const {
        return {
                FC_GetWidth (this->font->font_ptr, this->text.c_str()),
                FC_GetHeight(this->font->font_ptr, this->text.c_str())
        };
    }
    bool TextBase::GetWordWrap() const {return word_wrap;}
    void TextBase::SetWordWrap(bool wrap) { word_wrap = wrap; }
    void TextBase::SetText(std::string text) { this->text = text; }
    std::string TextBase::GetText() const { return this->text; }
    void TextBase::SetTextColor(SDL_Color c) { text_color = c; }
    SDL_Color TextBase::GetTextColor() const { return text_color; }
    void TextBase::SetHorizontalTextAlignment(TextAlignment::Horizontal align) { h_text_alignment = align; }
    void TextBase::SetVerticalTextAlignment(TextAlignment::Vertical align) { v_text_alignment = align; }
    TextAlignment::Vertical TextBase::GetVerticalTextAlignment() const { return v_text_alignment; }
    TextAlignment::Horizontal TextBase::GetHorizontalTextAlignment() const { return h_text_alignment; }
    void TextBase::SetTextOutlineColor(SDL_Color c) { outline_color = c; }
    SDL_Color TextBase::GetTextOutlineColor() const { return outline_color; }
    void TextBase::SetFont(JUI::Font* f) { font = f; }
    JUI::Font* TextBase::GetFont() const { return font; }
    void TextBase::Center() {
        SetHorizontalTextAlignment(TextAlignment::Horizontal::CENTER);
        SetVerticalTextAlignment(TextAlignment::Vertical::CENTER);
    }
    void TextBase::Draw(SDL_Renderer* target, v2f abs_pos, v2f abs_size) {
        float aligned_x = abs_pos.x;
        float aligned_y = abs_pos.y;
        FC_AlignEnum fc_align = FC_ALIGN_LEFT;

        Uint16 height = FC_GetHeight(this->font->font_ptr, this->text.c_str());

        if (h_text_alignment == TextAlignment::Horizontal::LEFT) {
            aligned_x = abs_pos.x;
            fc_align = FC_ALIGN_LEFT;
        }

        if (h_text_alignment == TextAlignment::Horizontal::CENTER) {
            aligned_x = abs_pos.x + (abs_size.x / 2);
            fc_align = FC_ALIGN_CENTER;
        }

        if (h_text_alignment == TextAlignment::Horizontal::RIGHT) {
            aligned_x = abs_pos.x + abs_size.x;
            fc_align = FC_ALIGN_RIGHT;
        }

        if (v_text_alignment == TextAlignment::Vertical::TOP) {
            aligned_y = abs_pos.y;
        }
        if (v_text_alignment == TextAlignment::Vertical::CENTER) {
            aligned_y = abs_pos.y + (abs_size.y / 2) - ((float) height / 2);
        }
        if (v_text_alignment == TextAlignment::Vertical::BOTTOM) {
            aligned_y = abs_pos.y + abs_size.y - height;
        }
        FC_SetDefaultColor(font->font_ptr, this->text_color);
        //SDL_SetRenderDrawColor(target, this->text_color.r, text_color.g, text_color.b, text_color.a);
        FC_DrawAlign(
                font->font_ptr,
                target,
                aligned_x,
                aligned_y,
                fc_align,
                text.c_str());
    }



}