#include <JUI/Widgets/LayoutContainers.hpp>

namespace JUI {


    JUI::LayoutContainerWidget::LayoutContainerWidget() {}

    JUI::LayoutContainerWidget::LayoutContainerWidget(Widget* parent) : LayoutContainerWidget() {
        this->SetParent(parent);
    }


    JUI::LayoutContainerWidget::~LayoutContainerWidget() {}

    void JUI::LayoutContainerWidget::Update(double delta) {
        ApplyLayout();
        Widget::Update(delta);
    }

    void JUI::LayoutContainerWidget::Draw(SDL_Renderer *target) {
        Widget::Draw(target);
    }

    void JUI::LayoutContainerWidget::ApplyLayout() {}


    void VerticalListLayout::Update(double delta) {
        LayoutContainerWidget::Update(delta);
    }

    void VerticalListLayout::Draw(SDL_Renderer *target) {
        LayoutContainerWidget::Draw(target);
    }

    void VerticalListLayout::ApplyLayout() {
        int consumed_height = 0;
        for (auto &child_widget: children) {
            // TODO: Implement widget.LayoutOrder property
            // TODO: Sort children by LayoutOrder
            consumed_height += padding.Pixels;
            child_widget->SetPosition({0, consumed_height, 0, 0});
            consumed_height += child_widget->GetAbsoluteSize().y;
        }
    }

    void HorizontalListLayout::Update(double delta) {
        LayoutContainerWidget::Update(delta);
    }

    void HorizontalListLayout::Draw(SDL_Renderer *target) {
        LayoutContainerWidget::Draw(target);
    }

    void HorizontalListLayout::ApplyLayout() {

        if (align == Align::LEFT)
        {
            int consumed_width = 0;
            for (auto &child_widget: children) {
                // TODO: Implement widget.LayoutOrder property
                // TODO: Sort children by LayoutOrder
                consumed_width += padding.Pixels;
                child_widget->SetPosition({consumed_width, 0, 0.0f, 0.0f});
                consumed_width += child_widget->GetAbsoluteSize().x;
            }
        }

        if (align == Align::RIGHT)
        {
            int consumed_width = this->GetAbsoluteSize().x;
            for (auto &child_widget: children) {
                // TODO: Implement widget.LayoutOrder property
                // TODO: Sort children by LayoutOrder
                consumed_width -= padding.Pixels;
                child_widget->SetPosition({consumed_width, 0, 0.0f, 0.0f});
                consumed_width -= child_widget->GetAbsoluteSize().x;
            }
        }


    }

    HorizontalListLayout::~HorizontalListLayout() {}
}
