#include <JUI/Widgets/Window.h>
#include "JUI/Widgets/Button.hpp"
#include "JUI/Widgets/LayoutContainers.hpp"


namespace JUI {

    Window::Window() : Rect(), DraggableElement()
    {
        this->SetPosition({200_px, 200_px});
        this->SetSize({400_px, 200_px});
        this->SetBackgroundColor({0, 0, 0, 0});

        this->MousePressed += [&] (MouseButtonID btn)
        {
            if (btn == MouseButtonID::LMB){
                if (draggable)
                    SetDrag(true);
            }

            if (btn == MouseButtonID::RMB)
            {
                if (resizable){

                }
            }

        };

        this->MouseReleased += [&] (MouseButtonID btn)
        {
            if (btn == MouseButtonID::LMB)
            {
                if (dragging)
                    SetDrag(false);
            }

            if (btn == MouseButtonID::RMB)
            {
                if (resizing)
                {

                }
            }

        };

        Viewport = new Rect(this);
        Viewport->SetName("Viewport");
        Viewport->SetBackgroundColor({64, 64, 64, 255});
        Viewport->SetSize({100_percent, 100_percent});
        Viewport->SetPosition({0_percent, 0_percent});
        Viewport->SetAnchorPoint({0.f, 0.f});
        Viewport->SetBorderColor({128, 128, 128, 255});
        Viewport->SetBorderWidth(1);
        Viewport->SetBorderRadius(0);

        Topbar = new Rect(Viewport);
        Topbar->SetPosition({0_px, 0_px});
        Topbar->SetSize({100_percent, 20_px});
        Topbar->SetBackgroundColor({128, 128, 128, 255});
        Topbar->SetBorderColor({128, 128, 128, 255});
        Topbar->SetBorderWidth(1);

        TitleLabel = new Text(Topbar);
        TitleLabel->Center();
        TitleLabel->SetText(title);

        auto* list = new HorizontalListLayout(Topbar);

        auto* exit_btn = new Button(list);
        exit_btn->SetAnchorPoint({1.f, 0.f});
        exit_btn->SetSize({30_px, 100_percent});
        exit_btn->SetBorderColor({128, 128, 128, 255});
        exit_btn->SetBaseColor({192, 64, 64, 255});
        exit_btn->SetHoverColor({255, 0, 0, 255});
        exit_btn->SetText("X");
        exit_btn->Center();

        exit_btn->MouseReleased += [&] (auto _) {
            this->SetVisible(false);
        };

        auto* fs_btn = new Button(list);
        fs_btn->SetAnchorPoint({1.f, 0.f});
        fs_btn->SetSize({30_px, 100_percent});
        fs_btn->SetBorderColor({128, 128, 128, 255});
        fs_btn->SetHoverColor({64, 64, 64, 255});
        fs_btn->SetBaseColor({128, 128, 128, 255});
        fs_btn->SetText("_");
        fs_btn->Center();


        fs_btn->MouseReleased += [&] (auto _)
        {
            this->SetSize({100_percent, 100_percent});
        };
    }

    Window::Window(Widget* parent) : Window()
    {
        this->SetParent(parent);
    }

    void Window::SetTitlebarHeight(int h) {
        titlebar_height = h;
        UDim height = {titlebar_height, 0.f};
        Topbar->SetSize({100_percent, height});
    }

    int Window::GetTitlebarHeight() const {
        return titlebar_height;
    }

    void Window::SetTitle(const std::string& new_title) {
        title = new_title;
        TitleLabel->SetText(title);
    }

    std::string Window::GetTitle() const { return title; }

    void Window::SetMaxDimensions(const v2i &dims) { max_dims = dims;}
    void Window::SetMinDimensions(const v2i &dims) {min_dims = dims; }
    v2i Window::GetMaxDimensions() const { return max_dims; }
    v2i Window::GetMinDimensions() const { return min_dims; }
    v2i Window::GetCurrentDimensions() const {
        return this->GetAbsoluteSize();
    }
    void Window::SetDraggable(bool enabled) { draggable = enabled; }
    void Window::SetResizable(bool enabled) { resizable = enabled;}

    void Window::Update(double delta) {
        Rect::Update(delta);

        if (dragging)
            this->SetPosition(UDim2{this->GetMouseCoordinates() - this->initial_drag_offset, {0.f,0.f}});
    }

    void Window::Draw(SDL_Renderer * target) {
        //if (!visible)
           // return;

        Rect::Draw(target);
    }

}