#include <JUI/common.h>
#include <SDL_image.h>

namespace JUI {

    Font* default_font;
    SDL_Texture* icon_checkmark;
    std::vector<Tween> ActiveTweens;

    Font* GetDefaultFont() { return default_font; }
    SDL_Texture* GetCheckmark() { return icon_checkmark; }

    void Load(SDL_Renderer *renderer) {
        default_font = new Font(renderer, "FreeSans.ttf", 12);
        icon_checkmark = IMG_LoadTexture(renderer, "check.png");
    }

    void Update(double delta) {

        // Remove any tween marked as dead
        ActiveTweens.erase(
            std::remove_if(ActiveTweens.begin(), ActiveTweens.end(),
            [&](Tween const& t)->bool { return t.done == true; }), ActiveTweens.end());

        for(auto& t : ActiveTweens) {
            t.Update(delta);
        }
    }

    void Cleanup() {

        SDL_DestroyTexture(icon_checkmark);
        delete default_font;
    }

    void draw_simple_text(SDL_Renderer *gfx, std::string text, JUI::v2i coords) {
        FC_SetDefaultColor(default_font->font_ptr, {255, 255, 255, 255});
        FC_DrawAlign(default_font->font_ptr, gfx, coords.x, coords.y, FC_ALIGN_LEFT, text.c_str());
    }

    void draw_simple_text(SDL_Renderer *gfx, std::string text, int x, int y) { draw_simple_text(gfx, text, {x, y}); }

    v2i GetMousePosition() {
        int x = 0;
        int y = 0;
        SDL_GetMouseState(&x, &y);
        return {x, y};
    }

    void SetWindowSize(SDL_Window *window, const JUI::v2i &size) { SetWindowSize(window, size.x, size.y); }

    void SetWindowSize(SDL_Window *window, int x, int y) { SDL_SetWindowSize(window, x, y); }

    v2i GetWindowSize(SDL_Window *window) {
        int x;
        int y;
        SDL_GetWindowSize(window, &x, &y);
        return {x, y};
    }

    v2i GetTextureDimensions(SDL_Texture *tex) {
        v2i size{0, 0};
        SDL_QueryTexture(tex, NULL, NULL, &size.x, &size.y);
        return size;
    }

    void ProcessSDLEvent(SDL_Event event) {
        if (event.type == SDL_MOUSEMOTION)      {MouseMoved   ({event.motion});}
        if (event.type == SDL_MOUSEBUTTONDOWN)  {MousePressed ({event.button});}
        if (event.type == SDL_MOUSEBUTTONUP)    {MouseReleased({event.button});}

    }

    void RunTween(JUI::Tween tween) {
        ActiveTweens.push_back(tween);
    }

    JUI::Angle operator ""_degrees(long double dg) { return {static_cast<float>(dg)}; }

    JUI::Angle operator ""_radians(long double rd) { return {static_cast<float>(rd)}; }

    JUI::Angle operator ""_deg(long double dg) { return {static_cast<float>(dg)}; }

    JUI::Angle operator ""_rad(long double rd) { return {static_cast<float>(rd)}; }

    Transform2D &Transform2D::Scale(const v2f &rhs) {
        return *this;
    }
}